import React, { Component } from 'react';
import { TextInput, Animated, Platform } from 'react-native';
import PropTypes from 'prop-types';
import _ from 'lodash';

import Font from './Font';

export default class Input extends Component {
  static propTypes = {
    placeholder: PropTypes.string,
    marginTop: PropTypes.number,
    validator: PropTypes.func,
    keyboardType: PropTypes.string,
    max: PropTypes.number,
    onChange: PropTypes.func,
    isSecure: PropTypes.bool,
    onFocus: PropTypes.func,
    onBlur: PropTypes.func,
    isDisabled: PropTypes.bool,
    type: PropTypes.oneOf([
      'text',
      'email',
      'number',
      'password'
    ]),
    value: PropTypes.string,
    placeholderColor: PropTypes.string,
    underlineColor: PropTypes.string,
    focusedPlaceholderColor: PropTypes.string,
    focusedUnderlineColor: PropTypes.string,
    selectionColor: PropTypes.string,
    errorColor: PropTypes.string,
    textColor: PropTypes.string,
    name: PropTypes.string
  };

  static defaultProps = {
    placeholder: '',
    marginTop: 0,
    validator: () => {},
    keyboardType: 'default',
    max: 50,
    onChange: () => {},
    isSecure: false,
    onFocus: () => {},
    onBlur: () => {},
    isDisabled: false,
    type: 'text',
    value: '',
    placeholderColor: '#3c4f5e',
    underlineColor: '#cfcfcf',
    focusedPlaceholderColor: '#cfcfcf',
    focusedUnderlineColor: '#4d4d4d',
    selectionColor: '#4d4d4d',
    errorColor: '#d0021b',
    textColor: '#4d4d4d',
    name: ''
  };

  state = {
    text: this.props.value || '',
    error: this.props.value ? '' : 'blank',
    overlaidBorderBottomStyle: {}
  };

  animatedValue = new Animated.Value(this.props.value ? 1 : 0);

  animatedTranslateY = this.animatedValue.interpolate({
    inputRange: [0, 1],
    outputRange: [0, -11]
  });

  animatedFontSize = this.animatedValue.interpolate({
    inputRange: [0, 1],
    outputRange: [14, 10]
  });

  animatedColor = this.animatedValue.interpolate({
    inputRange: [0, 1],
    outputRange: [this.props.placeholderColor, this.props.focusedPlaceholderColor]
  });

  animatedBorderBottomColor = this.animatedValue.interpolate({
    inputRange: [0, 1],
    outputRange: [this.props.underlineColor, this.props.focusedUnderlineColor]
  });

  componentWillReceiveProps(nextProps) {
    this.onChangeText(nextProps.value, 'skip onChange');

    if (nextProps.value) {
      this.animateUp();
    }
    else {
      if (this.textInputRef.isFocused()) {
        this.animateUp();
      }
      else {
        this.animateDown();
      }
    }
  }

  animateUp = _.debounce(() => {
    Animated.timing(this.animatedValue, {
      toValue: 1,
      duration: 200
    }).start();
  }, 200, {
    leading: true
  });

  animateDown = _.debounce(() => {
    Animated.timing(this.animatedValue, {
      toValue: 0,
      duration: 200
    }).start();
  }, 200, {
    leading: true
  });

  onFocus = () => {
    this.animateUp();

    this.setState({
      overlaidBorderBottomStyle: {}
    });

    this.props.onFocus();
  };

  onBlur = () => {
    if (!this.state.text) {
      this.animateDown();
    }

    this.setState({
      overlaidBorderBottomStyle: {
        borderBottomColor: this.props.underlineColor
      }
    });

    this.props.onBlur();
  };

  onChangeText = (text, option) => {
    let error = null;

    if (text) {
      error = this.props.validator(text);

      if (this.props.type == 'email') {
        if (!/^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/.test(text)) { // eslint-disable-line
          error = '이메일 형식이 올바르지 않습니다.';
        }
      }
      else if (this.props.type == 'password') {
        if (text.length < 6) {
          error = '비밀번호는 6자 이상으로 해주세요.';
        }
      }
    }
    else {
      error = 'blank';
    }

    this.setState({
      text: text || '\x02',
      error: error
    });

    if (!text) {
      setTimeout(() => {
        this.setState({
          text: text,
          error: error
        });
      }, 200);
    }

    if (option != 'skip onChange') {
      this.props.onChange(text, error || '', this.props.name);
    }
  };

  render() {
    let keyboardType = null;

    if (this.props.type == 'email') {
      keyboardType = 'email-address';
    }
    else if (this.props.type == 'number') {
      keyboardType = 'numeric';
    }
    else {
      keyboardType = this.props.keyboardType;
    }

    return (
      <Animated.View style={[{ justifyContent: 'flex-end', height: 50, borderBottomWidth: 1, borderBottomColor: this.animatedBorderBottomColor, marginTop: this.props.marginTop }, this.state.overlaidBorderBottomStyle ]}>
        <Animated.View style={{ position: 'absolute', top: 15, transform: [{ translateY: this.animatedTranslateY }], flexDirection: 'row' }}>
          <Animated.Text style={{ fontSize: this.animatedFontSize, color: this.animatedColor, fontFamily: Platform.OS == 'android' ? 'sans-serif' : 'Apple SD Gothic Neo' }}>
            { this.props.placeholder }
          </Animated.Text>
          { this.state.error != 'blank' && !!this.state.error &&
            <Font.H6 fontSize={10} color={this.props.errorColor} marginLeft={5}>{ this.state.error }</Font.H6>
          }
        </Animated.View>
        <TextInput
          ref={(ref) => { this.textInputRef = ref; }}
          onFocus={this.onFocus}
          onBlur={this.onBlur}
          onChangeText={this.onChangeText}
          style={{ height: 40, paddingTop: 10, color: this.props.textColor }}
          maxLength={this.props.max}
          keyboardType={keyboardType}
          selectionColor={this.props.selectionColor}
          secureTextEntry={this.props.type == 'password' ? true : this.props.isSecure}
          autoCapitalize="none"
          autoCorrect={false}
          editable={!this.props.isDisabled}
          value={this.state.text}
          underlineColorAndroid="transparent"
        />
      </Animated.View>
    );
  }
}
