import React, { Component } from 'react';
import { View, ScrollView, Dimensions, Animated, TouchableWithoutFeedback } from 'react-native';
import { getStatusBarHeight } from 'react-native-status-bar-height';
import PropTypes from 'prop-types';

import Font from './Font';

export default class Tutorial extends Component {
  static propTypes = {
    children: PropTypes.oneOfType([
      PropTypes.element,
      PropTypes.array
    ]).isRequired,
    backgroundColor: PropTypes.string,
    onPressSkip: PropTypes.func,
    skipText: PropTypes.string,
    skipColor: PropTypes.string,
    startText: PropTypes.string,
    startColor: PropTypes.string,
    bulletsColor: PropTypes.string,
    debounce: PropTypes.number
  };

  static defaultProps = {
    backgroundColor: '#ffffff',
    onPressSkip: () => {},
    skipText: '건너뛰기',
    skipColor: '#4d4d4d',
    startText: '시작하기',
    startColor: '#4d4d4d',
    bulletsColor: '#4d4d4d',
    debounce: 1000
  };

  state = {
    pageCount: this.props.children.length,
    isLastPage: false
  };

  width = Dimensions.get('window').width;

  animatedScrollX = new Animated.Value(0);

  animatedValue = this.animatedScrollX.interpolate({
    inputRange: [0, this.width * (this.state.pageCount - 1)],
    outputRange: [0, this.state.pageCount - 1]
  });

  animatedBulletTranslateX = this.animatedValue.interpolate({
    inputRange: [0, this.state.pageCount - 1],
    outputRange: [0 - 14 * (this.state.pageCount / 2 - 0.5), 14 * (this.state.pageCount / 2 - 0.5)]
  });

  renderPages = () => {
    return (
      this.props.children.map((page, index) => {
        return (
          <View key={index} style={{ flex: 1, width: this.width }}>
            { this.renderPageItems(page, index) }
          </View>
        );
      })
    );
  };

  renderPageItems = (page, index) => {
    const animatedPageTitleTranslateX = this.animatedValue.interpolate({
      inputRange: [index, index + 1],
      outputRange: [0, -100]
    });

    const animatedPageDescriptionTranslateX = this.animatedValue.interpolate({
      inputRange: [index, index + 1],
      outputRange: [0, -250]
    });

    return (
      <View style={{ flex: 1, alignItems: 'center' }}>
        <Animated.View style={{ transform: [{ translateX: animatedPageTitleTranslateX }] }}>
          { page.props.children[0] }
        </Animated.View>
        <Animated.View style={{ transform: [{ translateX: animatedPageDescriptionTranslateX }] }}>
          { page.props.children[1] }
        </Animated.View>
        { page.props.children[2] }
      </View>
    );
  };

  renderPagination = () => {
    return (
      <View style={{ flex: 1, flexDirection: 'row', alignItems: 'center', justifyContent: 'center' }}>
        <Animated.View style={{ transform: [{ translateX: this.animatedBulletTranslateX }], width: 4, height: 4, borderRadius: 2, backgroundColor: this.props.bulletsColor, position: 'absolute' }} />
        { this.renderBullets() }
      </View>
    );
  };

  renderBullets = () => {
    return (
      this.props.children.map((page, index) => {
        return (
          <View key={index} style={{ width: 4, height: 4, borderRadius: 2, borderColor: this.props.bulletsColor, borderWidth: 1, marginLeft: 5, marginRight: 5 }} />
        );
      })
    );
  };

  onScrollScrollView = () => {
    const animatedValue = this.animatedValue.__getValue();

    if (/\./.test(animatedValue + '')) {
      return;
    }

    if (animatedValue == this.state.pageCount - 1) {
      this.setState({
        isLastPage: true
      });
    }
    else {
      this.setState({
        isLastPage: false
      });
    }
  };

  isSkipPressed = false;

  onPressSkip = () => {
    if (this.isSkipPressed) {
      return;
    }

    this.isSkipPressed = true;

    setTimeout(() => {
      this.isSkipPressed = false;
    }, this.props.debounce);

    this.props.onPressSkip();
  };

  render() {
    return (
      <View style={{ flex: 1, backgroundColor: this.props.backgroundColor }}>
        <View style={{ height: getStatusBarHeight() }} />
        <View style={{ flex: 1 }}>
          <View style={{ height: 60, flexDirection: 'row', paddingHorizontal: 30 }}>
            <View style={{ flex: 1 }}>
              { !this.state.isLastPage &&
                <TouchableWithoutFeedback onPress={this.onPressSkip}>
                  <View style={{ flex: 1, alignItems: 'center', justifyContent: 'center' }}>
                    <Font.H4 color={this.props.skipColor}>{this.props.skipText}</Font.H4>
                  </View>
                </TouchableWithoutFeedback>
              }
            </View>
            <View style={{ flex: 1 }}>
              { this.renderPagination() }
            </View>
            <View style={{ flex: 1 }}>
              { this.state.isLastPage &&
                <TouchableWithoutFeedback onPress={this.onPressSkip}>
                  <View style={{ flex: 1, alignItems: 'center', justifyContent: 'center' }}>
                    <Font.H4 color={this.props.startColor}>{this.props.startText}</Font.H4>
                  </View>
                </TouchableWithoutFeedback>
              }
            </View>
          </View>
          <View style={{ flex: 1 }}>
            <ScrollView
              showsHorizontalScrollIndicator={false}
              horizontal
              pagingEnabled
              onScroll={Animated.event([
                { nativeEvent: {
                    contentOffset: {
                      x: this.animatedScrollX
                    }
                  }
                }
              ], {
                listener: this.onScrollScrollView
              })}
              scrollEventThrottle={1}
            >
              { this.renderPages() }
            </ScrollView>
          </View>
        </View>
      </View>
    );
  }
}
